//##############################################################################
// siphon
//
// bresenham.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include <stdio.h>

#include "bresenham.h"

//##############################################################################
// Exported functions
//##############################################################################
int bresenham_line_do(FIXED_POINT x1, FIXED_POINT y1, FIXED_POINT x2,
	FIXED_POINT y2, bresenham_callback callback, void * data)
{
	int retval = 0;
	int i = 0;

	FIXED_POINT x, y;
	FIXED_POINT dx, dy;
	FIXED_POINT incx, incy;
	FIXED_POINT balance;

	if (x2 >= x1)
	{
		dx = x2 - x1;
		incx = TOFIX(1);
	}
	else
	{
		dx = x1 - x2;
		incx = TOFIX(-1);
	}

	if (y2 >= y1)
	{
		dy = y2 - y1;
		incy = TOFIX(1);
	}
	else
	{
		dy = y1 - y2;
		incy = TOFIX(-1);
	}

	x = x1;
	y = y1;

	if (dx >= dy)
	{
		dy <<= 1;
		balance = dy - dx;
		dx <<= 1;

		for(i = 0; i < TOINT(dx) >> 1; i++)
		{
			if (balance >= TOFIX(0))
			{
				y += incy;
				balance -= dx;
			}
			balance += dy;
			x += incx;

			switch(callback(x, y, data))
			{
				case(NEXT):
					continue;
				case(REVERT_X):
					retval = 1;
					x -= incx;
					break;
				case(REVERT_Y):
					retval = 1;
					y -= incy;
					break;
				case(STOP):
					return 1;
			}
		}
	}
	else
	{
		dx <<= 1;
		balance = dx - dy;
		dy <<= 1;

		for(i = 0; i < TOINT(dy) >> 1; i++)
		{
			if (balance >= TOFIX(0))
			{
				x += incx;
				balance -= dy;
			}
			balance += dx;
			y += incy;

			switch(callback(x, y, data))
			{
				case(NEXT):
					continue;
				case(REVERT_X):
					retval = 1;
					x -= incx;
					break;
				case(REVERT_Y):
					y -= incy;
					retval = 1;
					break;
				case(STOP):
					return 1;
			}
		}
	}
	return retval;
}
