//##############################################################################
// siphon
//
// common.h
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#ifndef __COMMON_H__
#define __COMMON_H__

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#ifndef M_PI
	#define M_PI 3.14159265358979323846
#endif

typedef int bool;
#define FALSE 0
#define TRUE 1

#ifdef NDEBUG
	#define siphon_malloc(a) malloc(a)
	#define siphon_free(a) free(a)
	#define siphon_realloc(a, b) realloc(a, b)
	#define siphon_strdup(a) strdup(a)
	#define siphon_print_allocations() ;
#else
	#define STRINGIFY(x) #x
	#define TOSTRING(x) STRINGIFY(x)
	extern void * siphon_malloc_l(size_t bytes, const char *);
	extern void siphon_free(void * ptr);
	extern void * siphon_realloc(void * ptr, size_t size);
	extern char * siphon_strdup_l(const char * str, const char *);
	extern void siphon_print_allocations();
	#define siphon_malloc(a) siphon_malloc_l(a, __FILE__ ":" TOSTRING(__LINE__))
	#define siphon_strdup(a) siphon_strdup_l(a, __FILE__ ":" TOSTRING(__LINE__))
#endif

#if defined(_MSC_VER)
	#pragma section(".CRT$XCU",read)
	#define CONSTRUCTOR(f) \
	static void __cdecl f(); \
	__declspec(allocate(".CRT$XCU")) void (__cdecl*f##_)() = &f; \
	static void __cdecl f()
#else
	#define CONSTRUCTOR(f) static void __attribute__((constructor)) f()
#endif

static __inline int MAX(int a, int b)
{
	return a > b ? a : b;
}

static __inline int MIN(int a, int b)
{
	return a < b ? a : b;
}

static __inline float MAX_F(float a, float b)
{
	return a > b ? a : b;
}

static __inline float MIN_F(float a, float b)
{
	return a < b ? a : b;
}

static __inline void strip_whitespace(char * str_buffer)
{
	int length = strlen(str_buffer);
	int i = 0;
	for(i = 0; i < length; i++)
	{
		if(str_buffer[i] == '\n' || str_buffer[i] == '\r')
		{
			str_buffer[i] = '\0';
			break;
		}
	}
}

#endif
