//##############################################################################
// siphon
//
// config.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include <stdio.h>

#include "config.h"
#include "common.h"

//##############################################################################
// Static consts
//##############################################################################
static const char * CONFIG_FILE = "siphon_config.txt";

//##############################################################################
// Exported variables
//##############################################################################
Config config;

//##############################################################################
// Static functions
//##############################################################################
static int obfuscate_progress(int i)
{
	return (((i+20) * 1337523) ^ 123525023);
}

static int unobfuscate_progress(int i)
{
	int k = ((i ^ 123525023) / 1337523) - 20;
	if(k < 0 || k > 50 || obfuscate_progress(k) != i)
		return 0;
	return k;
}

//##############################################################################
// Exported functions
//##############################################################################
void config_initialize()
{
	FILE * file = NULL;
	int fullscreen;
	int music_enabled;
	int sample_enabled;

	// Set defaults
	memset(&config, 0x0, sizeof(Config));
	config.screen_width = -1;
	config.screen_height = -1;
	config.milliseconds_per_frame = 16;
	config.music_enabled = TRUE;
	config.sample_enabled = TRUE;

	// Load config file
	file = fopen(CONFIG_FILE, "r");
	if(file == NULL)
		return;

	fscanf(file, "%d\n", &fullscreen);
	fscanf(file, "%d\n", &config.progress);
	fscanf(file, "%d\n", &config.screen_width);
	fscanf(file, "%d\n", &config.screen_height);
	fscanf(file, "%d\n", &music_enabled);
	fscanf(file, "%d\n", &sample_enabled);

	config.progress = unobfuscate_progress(config.progress);
	config.fullscreen = fullscreen;
	config.music_enabled = music_enabled;
	config.sample_enabled = sample_enabled;

	fclose(file);
}

void config_save()
{
	FILE * file = fopen(CONFIG_FILE, "w");
	if(file == NULL)
	{
		fprintf(stdout, "Failed to create %s in working directory.\n",
			CONFIG_FILE);
		return;
	}

	fprintf(file, "%d\n", config.fullscreen);
	fprintf(file, "%d\n", obfuscate_progress(config.progress));
	fprintf(file, "%d\n", config.screen_width);
	fprintf(file, "%d\n", config.screen_height);
	fprintf(file, "%d\n", config.music_enabled);
	fprintf(file, "%d\n", config.sample_enabled);

	fclose(file);
}
