//##############################################################################
// siphon
//
// gameobject.h
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#ifndef __GAMEOBJECT_H__
#define __GAMEOBJECT_H__

#include "vector.h"
#include "map.h"

typedef enum
{
	PLAYER = 1 << 0,
	ROPE = 1 << 1,
	BONUS = 1 << 2,
	SPIKES = 1 << 3,
	FINISH = 1 << 4,
	SIGNPOST = 1 << 5
} GameObjectType;

typedef enum
{
	COLLISION_LEFT,
	COLLISION_RIGHT,
	COLLISION_UP,
	COLLISION_DOWN
} CollisionType;

typedef struct
{
	FixedVector pos;
	FixedVector vel;
} GameObjectHistory;

typedef struct _Event
{
	unsigned int tick;
	void * p1;
	void * p2;
	void (*undo_func)(void*,struct _Event *);
} Event;

typedef void (*EventUndoFunc)(void*,Event * event);

typedef struct GameObject
{
	GameObjectType type;
	Vector size;
	void (*logic)(struct GameObject*);
	void (*draw)(struct GameObject*, int, int);
	void (*destroy)(struct GameObject*);

	bool logic_updated;
	unsigned int movement_count;
	FixedVector movement[512];
	FixedVector pos;
	FixedVector vel;
	FixedVector req_pos;
	FixedVector req_vel;
	Map * map;

	unsigned int history_count;
	unsigned int history_size;
	GameObjectHistory * history;

	unsigned int events_count;
	unsigned int events_size;
	bool events_is_popping;
	Event * events;
	
} GameObject;

typedef struct GameObjectNode
{
	GameObject * go;
	struct GameObjectNode * next;
} GameObjectNode;

extern GameObjectNode * gameobject_create_node(GameObject * go);
extern void gameobject_append_node(GameObjectNode * list, GameObjectNode*);
extern void gameobject_destroy(GameObjectNode * node);
extern bool gameobject_collides(const GameObject * go, CollisionType type);
extern void gameobject_push_history(GameObject * go);
extern void gameobject_pop_history(GameObject * go);
extern void gameobject_push_event(GameObject * go, void*, void*, EventUndoFunc);
extern void gameobject_pop_event(GameObject * go);

#endif
