//##############################################################################
// siphon
//
// finish.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include <assert.h>
#include <SDL.h>

#include "finish.h"
#include "game.h"
#include "config.h"

//##############################################################################
// GameObject registration
//##############################################################################
CONSTRUCTOR(finish_const)
{
	registry_register("Finish", &finish_create);
}

//##############################################################################
// Static functions
//##############################################################################
static void finish_destroy(GameObject * vp_finish)
{
	Finish * finish = (Finish*)vp_finish;
	assert(vp_finish && vp_finish->type & FINISH);
	sprite_destroy(finish->sprite);
	siphon_free(finish);
}

static void finish_logic(GameObject * go)
{
	FixedVector gravity;
	int touch_ground = gameobject_collides(go, COLLISION_DOWN);
	int touch_left = gameobject_collides(go, COLLISION_LEFT);
	int touch_right = gameobject_collides(go, COLLISION_RIGHT);
	int touch_top = gameobject_collides(go, COLLISION_UP);

	go->req_pos = go->pos;
	go->req_vel = go->vel;

	gravity.x = TOFIX(0) / 4;
	gravity.y = TOFIX(1) / 4;

	if(config.flip_gravity)
	{
		gravity.y = -gravity.y;
		gravity.x = -gravity.x;
	}

	go->req_vel.x += gravity.x;
	go->req_vel.y += gravity.y;

	if(touch_ground)
		go->req_vel.y = MIN(go->req_vel.y, 0);
	if(touch_top)
		go->req_vel.y = MAX(go->req_vel.y, 0);
	if(touch_left)
		go->req_vel.x = MAX(go->req_vel.x, 0);
	if(touch_right)
		go->req_vel.x = MIN(go->req_vel.x, 0);

	go->req_pos.x += go->req_vel.x;
	go->req_pos.y += go->req_vel.y;
}

static void finish_draw(GameObject * vp_finish, int offset_x, int offset_y)
{
	Finish * finish = (Finish*)vp_finish;
	int b;
	assert(vp_finish && vp_finish->type & FINISH);

	b = (TOINT(vp_finish->pos.y) - offset_y) * 2 + vp_finish->size.y;

	if(config.flip_gravity)
	{
		glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
		glTranslatef(0.0f, -b, 0.0f);
	}

	sprite_draw(finish->sprite, TOINT(vp_finish->pos.x) - offset_x,
		TOINT(vp_finish->pos.y) - offset_y);

	if(config.flip_gravity)
	{
		glTranslatef(0.0f, b, 0.0f);
		glRotatef(-180.0f, 1.0f, 0.0f, 0.0f);
	}
}

//##############################################################################
// Exported functions
//##############################################################################
GameObject * finish_create(const CreateArgs * args)
{
	Finish * finish = (Finish*)siphon_malloc(sizeof(Finish));
	GameObject * gameobject = (GameObject*)&finish->gameobject;

	Vector position = properties_get_vector(args->properties, "position");

	memset(finish, 0x0, sizeof(Finish));
	gameobject->map = args->map;
	gameobject->pos = vector_to_fixed(position);
	gameobject->req_pos = vector_to_fixed(position);
	gameobject->type = FINISH;
	gameobject->size.x = 20;
	gameobject->size.y = 17;
	gameobject->logic = &finish_logic;
	gameobject->draw = &finish_draw;
	gameobject->destroy = &finish_destroy;

	finish->sprite = sprite_create("res/finish.txt");
	sprite_set_animation(finish->sprite, "Stand");

	if(args->addfunc != NULL)
		args->addfunc(gameobject);

	return NULL;
}
