//##############################################################################
// siphon
//
// spikes.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include <assert.h>
#include <SDL.h>

#include "spikes.h"
#include "game.h"
#include "config.h"

//##############################################################################
// GameObject registration
//##############################################################################
CONSTRUCTOR(spikes_const)
{
	registry_register("Spikes", &spikes_create);
}

//##############################################################################
// Static functions
//##############################################################################
static void spikes_destroy(GameObject * vp_spikes)
{
	Spikes * spikes = (Spikes*)vp_spikes;
	assert(vp_spikes && vp_spikes->type & SPIKES);
	sprite_destroy(spikes->sprite);
	siphon_free(spikes);
}

static void spikes_logic(GameObject * gameobject)
{
}

static void spikes_draw(GameObject * vp_spikes, int offset_x, int offset_y)
{
	Spikes * spikes = (Spikes*)vp_spikes;
	assert(vp_spikes && vp_spikes->type & SPIKES);

	sprite_draw(spikes->sprite, TOINT(vp_spikes->pos.x) - offset_x,
		TOINT(vp_spikes->pos.y) - offset_y);
}

//##############################################################################
// Exported functions
//##############################################################################
GameObject * spikes_create(const CreateArgs * args)
{
	Spikes * spikes = (Spikes*)siphon_malloc(sizeof(Spikes));
	GameObject * gameobject = (GameObject*)&spikes->gameobject;

	Vector position = properties_get_vector(args->properties, "position");

	memset(spikes, 0, sizeof(Spikes));
	gameobject->map = args->map;
	gameobject->pos = vector_to_fixed(position);
	gameobject->req_pos = vector_to_fixed(position);
	gameobject->type = SPIKES;
	gameobject->size.x = 12;
	gameobject->size.y = 3;
	gameobject->logic = &spikes_logic;
	gameobject->draw = &spikes_draw;
	gameobject->destroy = &spikes_destroy;

	spikes->sprite = sprite_create("res/spikes.txt");
	sprite_set_animation(spikes->sprite, "Stand");

	if(args->addfunc != NULL)
		args->addfunc(gameobject);

	return NULL;
}
