//##############################################################################
// siphon
//
// main.h
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#ifndef __MAIN_H__
#define __MAIN_H__

#include <SDL.h>

#include "common.h"

typedef void (*InitializationFunc)();
typedef void (*CleanupFunc)();
typedef void (*LogicFunc)();
typedef void (*DrawFunc)();
typedef void (*HandleEventFunc)(SDL_Event*);
typedef void (*EnterStateFunc)();
typedef void (*LeaveStateFunc)(const char * next_state);

typedef struct
{
	const char * name;
	InitializationFunc initialize;
	CleanupFunc cleanup;
	LogicFunc logic;
	DrawFunc draw;
	HandleEventFunc handle_event;
	EnterStateFunc enter_state;
	LeaveStateFunc leave_state;
} StateInterface;

extern void main_register_state(StateInterface interface);
extern void main_set_running(bool running);
extern void main_set_gamestate(const char * name);
extern DrawFunc main_get_drawfunc(const char * name);

#endif
