//##############################################################################
// siphon
//
// properties.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include <string.h>
#include <assert.h>

#include "properties.h"
#include "common.h"

//##############################################################################
// Exported functions
//##############################################################################
Properties * properties_create()
{
	Properties * properties = (Properties*)siphon_malloc(sizeof(Properties));
	properties->count = 0;
	properties->capacity = 2;
	properties->data = (Property*)siphon_malloc(
		sizeof(Property) * properties->capacity);

	return properties;
}

void properties_destroy(Properties * properties)
{
	unsigned int i = 0;

	assert(properties != NULL);
	
	for(i = 0; i < properties->count; i++)
	{
		siphon_free(properties->data[i].name);
		siphon_free(properties->data[i].value);
	}
	siphon_free(properties->data);
	siphon_free(properties);
}

void properties_add(Properties * properties, const char * name,
	const char * value)
{
	assert(properties != NULL);
	assert(name != NULL);
	assert(value != NULL);
	
	if(properties->count == properties->capacity)
	{
		properties->capacity *= 2;
		properties->data = (Property*)siphon_realloc(properties->data,
			sizeof(Property) * properties->capacity);
	}

	properties->data[properties->count].name = siphon_strdup(name);
	properties->data[properties->count].value = siphon_strdup(value);
	properties->count++;
}

const char * properties_get_value(Properties * properties, const char * name)
{
	unsigned int i = 0;

	assert(properties != NULL);
	assert(name != NULL);

	for(i = 0; i < properties->count; i++)
	{
		if(strcmp(properties->data[i].name, name) == 0)
			return properties->data[i].value;
	}

	fprintf(stderr, "Failed to locate property: '%s'\n", name);
	exit(EXIT_FAILURE);
	
	return NULL;
}

Vector properties_get_vector(Properties * properties, const char * name)
{
	Vector vec;
	const char * value = properties_get_value(properties, name);
	if(value == NULL)
	{
		fprintf(stderr, "Failed to locate property '%s'\n", name);
		exit(EXIT_FAILURE);
	}
	sscanf(value, "%d,%d", &vec.x, &vec.y);
	return vec;
}

int properties_get_int(Properties * properties, const char * name)
{
	int result = 0;
	const char * value = properties_get_value(properties, name);
	if(value == NULL)
	{
		fprintf(stderr, "Failed to locate property '%s'\n", name);
		exit(EXIT_FAILURE);
	}
	sscanf(value, "%d", &result);
	return result;
}

void properties_add_from_string(Properties * properties, char * buffer)
{
	int i = 0;
	
	// Read properties
	while(buffer[0])
	{
		char * property_name = NULL;
		char * property_value = NULL;

		// Read property name
		property_name = buffer;
		for(i = 0; buffer[i]; i++)
		{
			if(buffer[i] == '=')
			{
				if(buffer[i+1] == '(')
				{
					buffer[i] = '\0';
					buffer[i+1] = '\0';
					buffer = &buffer[i+2];
					break;
				}
				else
				{
					fprintf(stderr, "Property parsing error\n"
						"buffer contents: '%s'\n"
						"expected '('\n", buffer);
					exit(EXIT_FAILURE);
				}
			}
		}

		// Was the property name read?
		if(property_name == buffer)
		{
			fprintf(stderr, "Property parsing error\n"
				"buffer contents: '%s'\n", buffer);
			exit(EXIT_FAILURE);
		}

		// Read property value
		property_value = buffer;
		for(i = 0; buffer[i]; i++)
		{
			if(buffer[i] == ')')
			{
				buffer[i] = '\0';
				buffer = &buffer[i+1];
				break;
			}
			else if(buffer[i] == '(')
			{
				fprintf(stderr, "Property parsing error\n"
					"buffer contents: '%s'\n"
					"read '(' before ')' was encountered\n", buffer);
				exit(EXIT_FAILURE);
			}
		}

		// Was the property value read?
		if(property_value == buffer)
		{
			fprintf(stderr, "Property parsing error\n"
				"property name: '%s'\n"
				"buffer contents: '%s'\n", property_name, buffer);
			exit(EXIT_FAILURE);
		}

		// Skip over whitespace
		for(; buffer[0] == ' '; buffer++);

		properties_add(properties, property_name, property_value);
	}
}