//##############################################################################
// siphon
//
// registry.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include <assert.h>
#include <stdlib.h>

#include "registry.h"

//##############################################################################
// Linked List with void * value
//##############################################################################
typedef struct Node
{
	void * value;
	char * name;
	struct Node * next;
} Node;

static Node * get_node(Node * list, const char * name)
{
	while(list != NULL)
	{
		if(strcmp(list->name, name) == 0)
			return list;
		list = list->next;
	}
	return NULL;
}

static void append_node(Node * list, Node * node)
{
	assert(list != NULL);
	while(list->next != NULL)
	{
		list = list->next;
	}
	list->next = node;
}

static void delete_node(Node * node)
{
	assert(node != NULL);
	siphon_free(node->name);
	siphon_free(node);
}

static Node * create_node(const char * name, void * value)
{
	Node * node = (Node*)siphon_malloc(sizeof(Node));
	node->value = value;
	node->name = siphon_strdup(name);
	node->next = NULL;
	return node;
}

//##############################################################################
// Static variables
//##############################################################################
static Node * m_registered_gameobjects = NULL;

//##############################################################################
// Exported functions
//##############################################################################
void registry_create(const char * name, const CreateArgs * args)
{
	CreateFunc create_function;
	Node * node = NULL;

	assert(name);
	assert(args);
	assert(args->map);

	node = get_node(m_registered_gameobjects, name);
	if(node == NULL)
	{
		fprintf(stderr, "%s was not found in the gameobject registry\n", name);
		exit(EXIT_FAILURE);
	}
	else
	{
		create_function = (CreateFunc)node->value;
		create_function(args);
	}
}

bool registry_contains(const char * name)
{
	Node * node = get_node(m_registered_gameobjects, name);
	if(node == NULL)
		return FALSE;
	else
		return TRUE;
}

void registry_register(const char * name, CreateFunc func)
{
	static int first_registration = TRUE;

	if(first_registration)
	{
		m_registered_gameobjects = create_node(name, (void*)func);
		first_registration = FALSE;
	}
	else
	{
		Node * node = create_node(name, (void*)func);
		append_node(m_registered_gameobjects, node);
	}
}

void registry_unregister()
{
	Node * list = m_registered_gameobjects;
	while(list != NULL)
	{
		Node * next = list->next;
		delete_node(list);
		list = next;
	}
	m_registered_gameobjects = NULL;
}
