//##############################################################################
// siphon
//
// sprite.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <SDL.h>

#include "sprite.h"
#include "common.h"
#include "resources.h"

//##############################################################################
// Exported functions
//##############################################################################
Sprite * sprite_create(const char * name)
{
	Sprite * sprite = (Sprite*)siphon_malloc(sizeof(Sprite));
	assert(name != NULL);
	sprite->current_animation = NULL;
	sprite->info = resources_get_spriteinfo(name);
	sprite->frame_count = 0;

	return sprite;
}

void sprite_set_animation(Sprite * sprite, const char * name)
{
	int amount = 0;
	int i = 0;

	assert(sprite != NULL);
	assert(sprite->info != NULL);
	assert(name != NULL);

	if(sprite->current_animation != NULL &&
		strcmp(sprite->current_animation->name, name) == 0)
	{
		return;
	}

	amount = sprite->info->animation_count;

	for(i = 0; i < amount; i++)
	{
		SpriteAnimation * animation = &sprite->info->animations[i];
		if(strcmp(animation->name, name) == 0)
		{
			sprite->current_animation = animation;
			return;
		}
	}
	fprintf(stderr, "Failed to set animation %s\n", name);
	exit(EXIT_FAILURE);
}

void sprite_draw(Sprite * sprite, int x, int y)
{
	SpriteAnimation * anim = NULL;
	SpriteFrame * rect = NULL;
	int frameindex = 0;

	assert(sprite != NULL);
	assert(sprite->info != NULL);
	assert(sprite->info->tex != NULL);
	assert(sprite->current_animation != NULL);
	assert(sprite->current_animation->frames != NULL);

	if(SDL_GetKeyState(NULL)[SDLK_r])
		sprite->frame_count--;
	else
		sprite->frame_count++;

	anim = sprite->current_animation;

	if(anim == NULL)
	{
		// Draw simple rectangle to indicate no animation has been set
		glBegin(GL_QUADS);
		glVertex2f(x, y);
		glVertex2f(x + 16, y);
		glVertex2f(x + 16, y + 16);
		glVertex2f(x, y + 16);
		glEnd();
		return;
	}

	frameindex = (sprite->frame_count / anim->divisor) % anim->frame_count;

	rect = &anim->frames[frameindex];

	glBindTexture(GL_TEXTURE_2D, *sprite->info->tex);
	glBegin(GL_QUADS);
	glTexCoord2f(rect->left, rect->top);
	glVertex2f(x, y);

	glTexCoord2f(rect->right, rect->top);
	glVertex2f(x + rect->width, y);

	glTexCoord2f(rect->right, rect->bottom);
	glVertex2f(x + rect->width, y + rect->height);

	glTexCoord2f(rect->left, rect->bottom);
	glVertex2f(x, y + rect->height);
	glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);
}

void sprite_destroy(Sprite * sprite)
{
	assert(sprite != NULL);
	siphon_free(sprite);
}

void sprite_destroy_info(SpriteInfo * info)
{
	int i = 0;
	for(i = 0; i < info->animation_count; i++)
	{
		SpriteAnimation * anim = &info->animations[i];
		siphon_free(anim->name);
		siphon_free(anim->frames);
	}
	siphon_free(info->animations);
	siphon_free(info);
}
