//##############################################################################
// siphon
//
// intro.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include "intro.h"
#include "main.h"
#include "common.h"
#include "resources.h"
#include "viewport.h"
#include "vector.h"
#include "font.h"
#include "transition.h"
#include "levelselect.h"

//##############################################################################
// Static variables
//##############################################################################
static Font * m_rpg_font = NULL;
static Font * m_fixed_font = NULL;
static Font * m_small_font = NULL;
static int m_intro_ticks = 0;

//##############################################################################
// Static functions
//##############################################################################
static void intro_draw_background()
{
	Vector size = viewport_get_size();

	glBegin(GL_QUADS);
		glColor4f(0.0f, 0.15f, 0.47f, 1.0f);
		glVertex2f(0, 0);
		glColor4f(0.0f, 0.15f, 0.47f, 1.0f);
		glVertex2f(size.x, 0);
		glColor4f(0.39f, 0.76f, 1.0f, 1.0f);
		glVertex2f(size.x, size.y);
		glColor4f(0.39f, 0.76f, 1.0f, 1.0f);
		glVertex2f(0, size.y);
	glEnd();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

static void intro_enter_state()
{
	m_intro_ticks = -100;
}

static void intro_leave_state(const char * next_state)
{
	m_intro_ticks = -100;
}

static void intro_initialize()
{
	if(m_rpg_font == NULL)
		m_rpg_font = font_create("res/rpgfont.png", " abcdefghijklmnopqrstuvwxy"
			"zABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,!?-+/():;%&`'*#=[]\"");

	if(m_fixed_font == NULL)
		m_fixed_font = font_create("res/fixedfont.png", " abcdefghijklmnopqrstu"
			"vwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-.");

	if(m_small_font == NULL)
		m_small_font = font_create("res/small_font.png", " abcdefghijklmnopqrst"
			"uvwxyz-1234567890");
}

static void intro_handle_event(SDL_Event * event)
{
	switch(event->type)
	{
		case SDL_KEYDOWN:
			if(event->key.keysym.sym == SDLK_ESCAPE)
				main_set_running(FALSE);
			else
				transition_from_to("intro", "levelselect",
					TRANSITION_FADE_BLACK);
			break;
	}
}

static void intro_draw()
{
	const char * title = "Siphon";
	const char * subtitle = "written for the";
	const char * subtitle2 = "unintuitive physics competition";
	const char * subtitle3 = "at moosader.com";
	const char * credits = "by Maato";

	Vector textsize;
	Vector vec;
	Vector size = viewport_get_size();

	glBindTexture(GL_TEXTURE_2D, 0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	intro_draw_background();

	textsize = font_measure_text(m_rpg_font, title);
	vec.x = (size.x / 2 - textsize.x / 2);
	vec.y = m_intro_ticks + size.y / 2 - textsize.y / 2 - 24;
	font_draw_text(m_rpg_font, vec, title);

	textsize = font_measure_text(m_fixed_font, subtitle);
	vec.x = size.x / 2 - textsize.x / 2;
	vec.y = -m_intro_ticks + size.y / 2 - textsize.y / 2 + 0;
	font_draw_text(m_fixed_font, vec, subtitle);

	textsize = font_measure_text(m_fixed_font, subtitle2);
	vec.x = size.x / 2 - textsize.x / 2;
	vec.y = -m_intro_ticks + size.y / 2 - textsize.y / 2 + 12;
	font_draw_text(m_fixed_font, vec, subtitle2);

	textsize = font_measure_text(m_fixed_font, subtitle3);
	vec.x = size.x / 2 - textsize.x / 2;
	vec.y = -m_intro_ticks + size.y / 2 - textsize.y / 2 + 24;
	font_draw_text(m_fixed_font, vec, subtitle3);

	textsize = font_measure_text(m_fixed_font, credits);
	vec.x = size.x / 2 - textsize.x / 2;
	vec.y = -m_intro_ticks + size.y / 2 - textsize.y / 2 + 65;
	font_draw_text(m_fixed_font, vec, credits);
}

static void intro_logic()
{
	m_intro_ticks += 2;
	if(m_intro_ticks > 0)
		m_intro_ticks = 0;
}

static void intro_cleanup()
{
	font_destroy(m_rpg_font);
	font_destroy(m_fixed_font);
	font_destroy(m_small_font);
}

//##############################################################################
// State registration
//##############################################################################
CONSTRUCTOR(intro_const)
{
	StateInterface interface;
	memset(&interface, 0x0, sizeof(StateInterface));
	interface.name = "intro";
	interface.initialize = &intro_initialize;
	interface.cleanup = &intro_cleanup;
	interface.logic = &intro_logic;
	interface.draw = &intro_draw;
	interface.handle_event = &intro_handle_event;
	interface.enter_state = &intro_enter_state;
	interface.leave_state = &intro_leave_state;

	main_register_state(interface);
}
