//##############################################################################
// siphon
//
// paused.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include "paused.h"
#include "main.h"
#include "viewport.h"
#include "vector.h"
#include "font.h"
#include "game.h"

//##############################################################################
// Static variables
//##############################################################################
static Font * m_rpg_font = NULL;
static int m_draw_ticks = 0;

//##############################################################################
// Static functions
//##############################################################################
static void paused_enter_state()
{
	m_draw_ticks = -100;
}

static void paused_initialize()
{
	if(m_rpg_font == NULL)
		m_rpg_font = font_create("res/rpgfont.png", " abcdefghijklmnopqrstuvwxy"
			"zABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,!?-+/():;%&`'*#=[]\"");
}

static void paused_handle_event(SDL_Event * event)
{
	switch(event->type)
	{
		case SDL_KEYDOWN:
			if(SDL_GetModState() & (KMOD_LSHIFT | KMOD_RSHIFT | KMOD_LCTRL |
				KMOD_RCTRL | KMOD_LALT | KMOD_RALT))
			{
				return;
			}
			if(event->key.keysym.sym == SDLK_p)
				return;
			main_set_gamestate("game");
			break;
	}
}

static void paused_logic()
{
}

static void paused_draw()
{
	Vector size = viewport_get_size();
	const char * title = "Game Paused";
	Vector vec;
	Vector textsize = font_measure_text(m_rpg_font, title);

	main_get_drawfunc("game")();

	m_draw_ticks += 2;
	if(m_draw_ticks > 0)
		m_draw_ticks = 0;

	vec.x = (size.x / 2 - textsize.x / 2);
	vec.y = m_draw_ticks + size.y / 2 - textsize.y / 2 - 12;
	font_draw_text(m_rpg_font, vec, title);
}

static void paused_cleanup()
{
	font_destroy(m_rpg_font);
}

static void paused_leave_state(const char * next_state)
{
}

//##############################################################################
// State registration
//##############################################################################
CONSTRUCTOR(paused_const)
{
	StateInterface interface;
	memset(&interface, 0x0, sizeof(StateInterface));
	interface.name = "paused";
	interface.initialize = &paused_initialize;
	interface.cleanup = &paused_cleanup;
	interface.logic = &paused_logic;
	interface.draw = &paused_draw;
	interface.handle_event = &paused_handle_event;
	interface.enter_state = &paused_enter_state;
	interface.leave_state = &paused_leave_state;

	main_register_state(interface);
}
