//##############################################################################
// siphon
//
// transition.c
//
// Copyright 2010 Maato
//
// Authors:
//    Maato <maato@softwarebakery.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//##############################################################################
#include <assert.h>

#include "transition.h"
#include "main.h"
#include "vector.h"
#include "viewport.h"
#include "opengl_helper.h"

//##############################################################################
// Static variables
//##############################################################################
static DrawFunc m_old_draw_func = NULL;
static DrawFunc m_new_draw_func = NULL;
static const char * m_new_gamestate = NULL;
static TransitionType m_transition_type = -1;
static int m_tick_count = 0;

//##############################################################################
// Static functions
//##############################################################################
static void slide_right()
{
	Vector size = viewport_get_size();

	float x = (float)size.x / (30.0f * 1.0f);
	x *= m_tick_count;

	glTranslatef((float)x, 0.0f, 0.0f);
	m_old_draw_func();
	glTranslatef((float)-x, 0.0f, 0.0f);

	viewport_scissor_start(0, 0, x, size.y);
	glTranslatef(x - size.x, 0.0f, 0.0f);
	m_new_draw_func();
	glTranslatef(-(x - size.x), 0.0f, 0.0f);
	viewport_scissor_stop();
}

static void fade_black()
{
	Vector size = viewport_get_size();

	float percentage = m_tick_count / 15.0f;

	if(percentage >= 1.0f)
	{
		percentage = 2.0f - percentage;
		m_new_draw_func();
	}
	else
	{
		m_old_draw_func();
	}

	glColor4f(0.0f, 0.0f, 0.0f, percentage);
	glBegin(GL_QUADS);
	glVertex2f(0.0f, 0.0f);
	glVertex2f(size.x, 0.0f);
	glVertex2f(size.x, size.y);
	glVertex2f(0.0f, size.y);
	glEnd();
}

static void transition_initialize()
{
}

static void transition_enter_state()
{
	assert(m_old_draw_func != NULL);
	assert(m_new_draw_func != NULL);
	assert(m_old_draw_func != m_new_draw_func);
	assert(m_new_gamestate != NULL);
	assert(m_transition_type != -1);
	m_tick_count = 0;
}

static void transition_handle_event(SDL_Event * event)
{
}

static void transition_logic()
{
	m_tick_count++;
	if(m_tick_count >= 30 * 1)
		main_set_gamestate(m_new_gamestate);
}

static void transition_draw()
{
	switch(m_transition_type)
	{
		case TRANSITION_SLIDE_RIGHT:
			slide_right();
			break;
		default:
			fade_black();
			break;
	}
}

static void transition_cleanup()
{
}

static void transition_leave_state(const char * next_state)
{
}

//##############################################################################
// Exported functions
//##############################################################################
void transition_from_to(const char * from, const char * to, TransitionType type)
{
	m_old_draw_func = main_get_drawfunc(from);
	m_new_draw_func = main_get_drawfunc(to);
	m_new_gamestate = to;
	m_transition_type = type;
	main_set_gamestate("transition");
}

//##############################################################################
// State registration
//##############################################################################
CONSTRUCTOR(transition_const)
{
	StateInterface interface;
	memset(&interface, 0x0, sizeof(StateInterface));
	interface.name = "transition";
	interface.initialize = &transition_initialize;
	interface.cleanup = &transition_cleanup;
	interface.logic = &transition_logic;
	interface.draw = &transition_draw;
	interface.handle_event = &transition_handle_event;
	interface.enter_state = &transition_enter_state;
	interface.leave_state = &transition_leave_state;

	main_register_state(interface);
}
